#!/bin/sh
#


# von Dieter Kroemer (31.08.03):
# Vorgehensweise:
#    1.	Backupplatte z.B. mit Yast2 nach /root/snapshot mounten und dafuer 
#	sorgen, dass diese Platte auch nach einem Reboot dort eingehangen wird.
#    2. Die Variablen weiter unten entsprechend Ihren Voraussetzungen
#	anpassen.
#
#
# ----------------------------------------------------------------------
# Script basiert auf mikes handy rotating-filesystem-snapshot utility
# http://linuxwiki.de/rsync/SnapshotBackups
# ----------------------------------------------------------------------
# RCS info: $Id: rsync_2fSnapshotBackups,v 1.16 2003/01/16 02:43:23 linuxwiki_de Exp $
# ----------------------------------------------------------------------


## ----Vom User zu editierende Variablen:----

backupadmin=kroe          	# dieser User erhlt Backup-Email 

PROZENT=96                	# max. Plattenbelegung (in %) der Backupplatte 

NR=60                 		# max. Anzahl der Sicherungen
 	
MOUNT_DEVICE=/dev/hdb1 		# Deviceangabe der Backupfestplatte

SNAPSHOT_RW=/root/snapshot	# Mountpoint der Backup-Platte (so lassen)



## ----- ab hier sollte nichts mehr editiert werden muessen!! -----




NRLN=$NR # Fuer die Links im Homeverzeichnis!!
NR_RM=$NR #Zum Loeschen der alten Snapshots

#Plattenbelegung in % wird ermittelt:
alias platz="df $MOUNT_DEVICE|tail -n 1|awk -F'%' '{print \$1}'|awk -F' ' '{print \$5}'"
PLATZ=`platz`
alias giga="df -h $MOUNT_DEVICE|tail -n 1|awk -F' ' '{print \$3 \" von (\" \$2 \")\" }'"
# ------------- system commands used by this script --------------------
ID=/usr/bin/id;
ECHO=/bin/echo;
CAT=/bin/cat

MOUNT=/bin/mount;
RM=/bin/rm;
MV=/bin/mv;
MKDIR=/bin/mkdir
CP=/bin/cp;
LN=/bin/ln
TOUCH=/bin/touch;

RSYNC=/usr/bin/rsync;

DATE=/bin/date

# ------------- file locations -----------------------------------------
EXCLUDES=/usr/local/etc/backup_exclude;
RELATIV_SNAPSHOT_RW=${SNAPSHOT_RW#/*} #zum Ausschluss von root/snapshot beim Sichern

# ------------- the script itself --------------------------------------

$DATE > /tmp/backup.tmp
$ECHO "Das Backupscript beginnt:">> /tmp/backup.tmp

$ECHO "Ihre Festplatte ist zu $PLATZ% (=`giga`) belegt" >>/tmp/backup.tmp
$ECHO " " >> /tmp/backup.tmp

# make sure we're running as root
if (( `$ID -u` != 0 )); then { $ECHO "Sorry, must be root.  Exiting..."; exit; } fi

# attempt to remount the RW mount point as RW; else abort
$MOUNT -o remount,rw $MOUNT_DEVICE $SNAPSHOT_RW ;
if (( $? )); then
{
        $ECHO "snapshot: could not remount $SNAPSHOT_RW readwrite" >> /tmp/backup.tmp
        exit;
}
fi;



## Die aeltesten Snapshots werden geloescht,
## bis die Backupplatte wieder genuegende Platz besitzt.

while [ $PLATZ -ge $PROZENT -a ! $NR_RM -eq 0 ] ; do


    if [ -d $SNAPSHOT_RW/tag.$NR_RM ] ; then
     $ECHO "Snapshot: tag.$NR_RM wird geloescht." >> /tmp/backup.tmp
     $RM -R $SNAPSHOT_RW/tag.$NR_RM
     PLATZ=`platz`
     $ECHO "Ihre Festplatte ist zu $PLATZ% (=`giga`) belegt." >> /tmp/backup.tmp
     $ECHO " " >> /tmp/backup.tmp
    fi;

  NR_RM=`expr $NR_RM - 1`


done




# rotating snapshots of / "alles wird gesichert!!"


# step 1: delete the oldest snapshot, if it exists:
if [ -d $SNAPSHOT_RW/tag.$NR ] ; then
$RM -rf $SNAPSHOT_RW/tag.$NR ;
fi ;

# step 2: shift the middle snapshots(s) back by one, if they exist

while [ $NR -gt 1 ] ; do

NR=`expr $NR - 1`
$ECHO $NR

if [ -d $SNAPSHOT_RW/tag.$NR ] ; then
$MV $SNAPSHOT_RW/tag.$NR $SNAPSHOT_RW/tag.`expr $NR + 1`;
fi; 

done 



# step 3: make a hard-link-only (except for dirs) copy of the latest snapshot, 
# if that exists 
if [ -d $SNAPSHOT_RW/tag.0 ] ; then
$CP -alv $SNAPSHOT_RW/tag.0 $SNAPSHOT_RW/tag.1 ;
fi; 
 
# step 4: rsync from the system into the latest snapshot (notice that 
# rsync behaves like cp --remove-destination by default, so the destination 
# is unlinked first.  If it were not so, this would copy over the other 
# snapshot(s) too!

if [ ! -d $SNAPSHOT_RW/tag.0 ] ; then
 $MKDIR $SNAPSHOT_RW/tag.0
fi

$RSYNC -va --delete --exclude=var --exclude=home --exclude=proc --exclude=snapshot --exclude=$RELATIV_SNAPSHOT_RW / $SNAPSHOT_RW/tag.0 ;
$RSYNC  -va --delete --delete-excluded   /home/ $SNAPSHOT_RW/tag.0/home/ ;
$RSYNC  -va --delete --delete-excluded   /var/ $SNAPSHOT_RW/tag.0/var/ ;


if [ ! -d $SNAPSHOT_RW/tag.0/proc ] ; then
  $MKDIR $SNAPSHOT_RW/tag.0/proc ;
fi;

if [ ! -d $SNAPSHOT_RW/home ] ; then
  $MKDIR $SNAPSHOT_RW/home
fi


# In das Verzeichnis $SNAPSHOT_RW/home werden Links zu $SNAPSHOT_RW/tag.*/home erstellt.
while [ $NRLN -ge 0 ] ; do
$ECHO $NRLN
  if [ -d $SNAPSHOT_RW/tag.$NRLN/home -a ! -L $SNAPSHOT_RW/home/home_alt.$NRLN ] ; then
    $LN -s ../tag.$NRLN/home $SNAPSHOT_RW/home/home_alt.$NRLN;
   fi;

if [ -L $SNAPSHOT_RW/home/home_alt.$NRLN -a ! -d $SNAPSHOT_RW/tag.$NRLN/home ] ; then
   $RM $SNAPSHOT_RW/home/home_alt.$NRLN;
fi;

NRLN=`expr $NRLN - 1`

done


# step 5: update the mtime of tag.0 to reflect the snapshot time
$TOUCH $SNAPSHOT_RW/tag.0 ;

# and thats it.

# now remount the RW snapshot mountpoint as readonly 
 
$MOUNT -o remount,ro $MOUNT_DEVICE $SNAPSHOT_RW ; 
if (( $? )); then 
{ 
        $ECHO "snapshot: could not remount $SNAPSHOT_RW readonly"; 
        exit; 
} fi; 


# Zum Schluss der Sicherung noch einige Infos fuers obligatorische mail.
PLATZ=`platz`
$ECHO " " >> /tmp/backup.tmp
$ECHO "Ihre Festplatte ist nach der Sicherung zu $PLATZ% (=`giga`) belegt" >> /tmp/backup.tmp
$DATE >> /tmp/backup.tmp
$CAT /tmp/backup.tmp |mail $backupadmin -s backup
