#!/bin/bash
# Script um im Rescue System einen alten Systemzustand wiederherszustellen
#
# erstellt von Dieter Kroemer, 16.08.03
# mailto: kroe@rs-schesslitz.de
#
#
# Evtl. muss man natuerlich bei einer neuen Platte die Partionen erst wieder erstellen.
# Dazu kann man auch mit der Standard-Installation beginnen, diese aber nach Erstellen
# der Partitionen beenden.
# Oder natrlich per Hand mit Hilfe von 'fdisk /dev/...'


## ----- Vom User zu editierende Variablen:------------------------

BACKUP_DEVICE=/dev/hdb1		# Partition, auf dem das Backup (/root/snapshot) liegt.

VAR_DEVICE=/dev/hda1		# /var-Partition des wiederherzustellenden System
SLASH_DEVICE=/dev/hda2		# /-Partition des wiederherzustellenden Systems
HOME_DEVICE=/dev/hda4		# /home-Partition des wiederherzustellenden Systems

SYSTEMZUSTAND=0			# Wiederherzustellender Systemzustand.
				# z.B. SYSTEMZUSTAND=0 letzter gesicherte Systemzustand
				#      SYSTEMZUSTAND=10 der vor 11 Sicherungen erstellte Systemzustand


## ----- ab hier sollte nichts mehr editiert werden muessen!! -----



#----Befehle-----

MOUNT=`which mount`
MKDIR=`which mkdir`
RSYNC=`which rsync`


#--- Im Folgenden werden die noetigen Mountpoints erstellt und die Partionen dorthin gemountet: ---

$MKDIR /root/backup
$MOUNT $BACKUP_DEVICE /root/backup

$MKDIR /root/restore
$MOUNT $SLASH_DEVICE /root/restore

$MKDIR /root/restore/var
$MKDIR /root/restore/home

$MOUNT $VAR_DEVICE /root/restore/var
$MOUNT $HOME_DEVICE /root/restore/home


# Im Folgenden wird der Systemzustand (hoffentlich ;-) ) wiederhergestellt:

$RSYNC -va --delete --exclude=var --exclude=home /root/backup/tag.$SYSTEMZUSTAND/ /root/restore/
$RSYNC -va --delete /root/backup/tag.$SYSTEMZUSTAND/var/ /root/restore/var/
$RSYNC -va --delete /root/backup/tag.$SYSTEMZUSTAND/home/ /root/restore/home/


#------Nach dem Ablauf des Scriptes:------
#-----------------------------------------
# Jetzt muss noch der Bootmanager (grub bzw. lilo) in den MBR geschrieben werden.
#
# Mit SCSI-Platten ohne Raid-System habe ich das noch nicht ausprobieren koennen :-(
# Aber entweder die grub-Vorgehensweise: grub-install .../dev/sda, oder die fuer lilo.
#
#
# grub (fuer IDE-Festplatten):
#
#	grub-install --root-directory=/root/restore /dev/hda
#	reboot
#
# lilo (fuer Raid-Systemen):
# da es Probleme bzgl. des Boot-Devices zwischen dem Bios und lilo gibt,  boote ich der Einfachheit
# halber nocheinmal das Rescue-System neu, dabei schalte ich aber zuvor _alle_!! Festplatten aus, die
# _nicht_ zum Raid-System gehoeren.
# Dann muss das $SLASH-DEVICE (z.B. /dev/ataraid/d0p6) nach /mnt gemountet
#
#	mount /dev/ataraid/d0p6 /mnt
#	chroot /mnt
#	lilo (der lilo wird in den MBR geschrieben)
#	exit (das ursprngliche root-Verzeichnis kommt wieder an seinen Ort)
#	reboot
#
#	Anm.: chroot ->	jetzt wird die root-Partition des Rescue-Systems von der zukuenftigen
#			root-Partion uebernommen; wichtig, da ja hier die ganzen Einstellungen
#			fuer lilo liegen)
#
# Und eigentlich muesste das System jetzt wieder startklar sein :-)
#
